# Extracted from theme js files on https://echarts.apache.org/en/download-theme.html

# Pass these values for `register_theme_code` prop, then set the theme name in
# the `theme` prop.

vintage = """
var colorPalette = [
    '#d87c7c',
    '#919e8b',
    '#d7ab82',
    '#6e7074',
    '#61a0a8',
    '#efa18d',
    '#787464',
    '#cc7e63',
    '#724e58',
    '#4b565b'
];
echarts.registerTheme('vintage', {
    color: colorPalette,
    backgroundColor: '#fef8ef',
    graph: {
        color: colorPalette
    }
});"""


roma = """
    var colorPalette = [
        '#E01F54',
        '#001852',
        '#f5e8c8',
        '#b8d2c7',
        '#c6b38e',
        '#a4d8c2',
        '#f3d999',
        '#d3758f',
        '#dcc392',
        '#2e4783',
        '#82b6e9',
        '#ff6347',
        '#a092f1',
        '#0a915d',
        '#eaf889',
        '#6699FF',
        '#ff6666',
        '#3cb371',
        '#d5b158',
        '#38b6b6'
    ];

    var theme = {
        color: colorPalette,

        visualMap: {
            color: ['#e01f54', '#e7dbc3'],
            textStyle: {
                color: '#333'
            }
        },

        candlestick: {
            itemStyle: {
                color: '#e01f54',
                color0: '#001852'
            },
            lineStyle: {
                width: 1,
                color: '#f5e8c8',
                color0: '#b8d2c7'
            },
            areaStyle: {
                color: '#a4d8c2',
                color0: '#f3d999'
            }
        },

        graph: {
            itemStyle: {
                color: '#a4d8c2'
            },
            linkStyle: {
                color: '#f3d999'
            }
        },

        gauge: {
            axisLine: {
                lineStyle: {
                    color: [
                        [0.2, '#E01F54'],
                        [0.8, '#b8d2c7'],
                        [1, '#001852']
                    ],
                    width: 8
                }
            }
        }
    };

    echarts.registerTheme('roma', theme);
"""



"""Wrapper for echarts-for-react library."""

from typing import Dict, Optional

import reflex as rx


class Echarts(rx.Component):
    """ReactECharts component.

    Usage:
        Set any echarts option dict to the option prop. The chart will update when the
        option data updates.

    See https://www.npmjs.com/package/echarts-for-react for explanation of props.
    See https://echarts.apache.org/examples/en/index.html for echarts examples.
    """

    library = "echarts-for-react@^3.0.2"

    # The React component tag.
    tag = "ReactECharts"

    is_default = True

    # the echarts option config, can see https://echarts.apache.org/option.html#title.
    option: rx.Var[Dict]

    # when setOption, not merge the data, default is false. See https://echarts.apache.org/api.html#echartsInstance.setOption.
    not_merge: rx.Var[bool]

    # when setOption, lazy update the data, default is false. See https://echarts.apache.org/api.html#echartsInstance.setOption.
    lazy_update: rx.Var[bool]

    # the echarts loading option config, can see https://echarts.apache.org/api.html#echartsInstance.showLoading.
    loading_option: rx.Var[Dict]

    # bool, when the chart is rendering, show the loading mask.
    show_loading: rx.Var[bool]

    # the theme of echarts. string
    theme: rx.Var[str]

    # The code that should be executed to register a theme
    register_theme_code: Optional[str] = None

    # the opts of echarts. object, will be used when initial echarts instance by echarts.init. Document here.
    opts: rx.Var[Dict]

    # Fired when the chart is ready
    on_chart_ready: rx.EventHandler[rx.event.no_args_event_spec]

    @classmethod
    def create(cls, *children, **props):
        """Create a ReactECharts component.

        Args:
            *children: The children of the component.
            **props: The properties of the component.

        Returns:
            The ReactECharts component.

        """
        props["width"] = props.pop("width", "100%")
        return super().create(*children, **props)

    def _exclude_props(self) -> list[str]:
        return [*super()._exclude_props(), "register_theme_code"]

    def add_imports(self) -> rx.ImportDict:
        return {
            "echarts": [rx.ImportVar(tag="* as echarts", is_default=True)],
        }

    def add_custom_code(self) -> list[str]:
        if self.register_theme_code is not None:
            return [self.register_theme_code]
        return []


echarts = Echarts.create



import json
import random
from pathlib import Path

import reflex as rx
from reflex_echarts import echarts

from . import themes

# https://echarts.apache.org/examples/data/asset/data/life-expectancy-table.json
with (Path(__file__).parent / "life-expectancy-table.json").open("r") as f:
    _raw_data = json.load(f)


class State(rx.State):
    """The app state."""

    line_chart: dict = {
        "xAxis": {
            "type": "category",
            "data": ["Mon", "Tue", "Wed", "Thu", "Fri", "Sat", "Sun"],
        },
        "yAxis": {"type": "value"},
        "series": [{"data": [150, 230, 224, 218, 135, 147, 260], "type": "line"}],
    }

    country_income_race: dict = {}

    @rx.event
    def munge_line_chart(self):
        self.line_chart["series"][0]["data"] = [
            random.randint(120, 280) for _ in range(7)
        ]

    @rx.event
    def on_load(self):
        countries = [
            "Finland",
            "France",
            "Germany",
            "Iceland",
            "Norway",
            "Poland",
            "Russia",
            "United Kingdom",
        ]
        dataset_with_filters = []
        series_list = []
        for country in countries:
            dataset_id = "dataset_" + country
            dataset_with_filters.append(
                {
                    "id": dataset_id,
                    "fromDatasetId": "dataset_raw",
                    "transform": {
                        "type": "filter",
                        "config": {
                            "and": [
                                {"dimension": "Year", "gte": 1950},
                                {"dimension": "Country", "=": country},
                            ]
                        },
                    },
                }
            )
            series_list.append(
                {
                    "type": "line",
                    "datasetId": dataset_id,
                    "showSymbol": False,
                    "name": country,
                    "endLabel": {
                        "show": True,
                    },
                    "labelLayout": {"moveOverlap": "shiftY"},
                    "emphasis": {"focus": "series"},
                    "encode": {
                        "x": "Year",
                        "y": "Income",
                        "label": ["Country", "Income"],
                        "itemName": "Year",
                        "tooltip": ["Income"],
                    },
                }
            )

        self.country_income_race = {
            "animationDuration": 10000,
            "dataset": [
                {"id": "dataset_raw", "source": _raw_data},
                *dataset_with_filters,
            ],
            "title": {"text": "Income of European countries since 1950"},
            "tooltip": {"order": "valueDesc", "trigger": "axis"},
            "xAxis": {"type": "category", "nameLocation": "middle"},
            "yAxis": {"name": "Income"},
            "grid": {"right": 140},
            "series": series_list,
        }


def index() -> rx.Component:
    return rx.vstack(
        echarts(option=State.line_chart, theme="dark"),
        rx.button("New Data for Line Chart", on_click=State.munge_line_chart),
        rx.divider(),
        echarts(
            option=State.country_income_race,
            register_theme_code=themes.roma,
            theme="roma",
            on_chart_ready=rx.console_log("Country income chart is ready"),
        ),
        align="center",
        width="100vw",
    )


# Add state and page to the app.
app = rx.App()
app.add_page(index, on_load=State.on_load)
